/** @file
 * @brief PadicoTM RT Task manager
 * @ingroup TaskManager
 */

#include <Padico/Puk.h>
#include <Padico/Module.h>

#include "PM2.h"
#include "TaskManager-internals.h"

PADICO_MODULE_HOOK(PadicoTM);




/* *** RTTASK MANAGER ************************************** */

void padico_tm_rttask_submit(padico_rttask_t rttask)
{
  padico_trace("enqueue RTTASK_REGISTER\n");
  switch(rttask->kind)
    {
    
    case PADICO_RTTASK_RTTHREAD:
      {
	marcel_t       background_thread;
	marcel_attr_t  attr;
	padico_trace("starting RT thread\n");
	marcel_attr_init(&attr);
	marcel_attr_setrealtime(&attr, MARCEL_CLASS_REALTIME);
	marcel_attr_setdetachstate(&attr, 1);
	marcel_create(&background_thread, &attr, 
		      rttask->background_fun, 
		      rttask->background_arg);
	padico_tm_thread_givename_tid("PadicoTM: RT-Task", background_thread);
      }
      break;
    
    case PADICO_RTTASK_BGTHREAD:
      {
	padico_trace("starting BG thread\n");
	/*
	  marcel_t       background_thread;
	  marcel_create(&background_thread, NULL, 
	  rttask->background_fun, 
	  rttask->background_arg); 
	*/
	padico_tm_invoke_later_named(rttask->background_fun, 
				     rttask->background_arg,
				     "PadicoTM: BG-Task");
      }
      break;
    }
}

void padico_tm_invoke_later_named(padico_thread_fun_t f, void*key, const char*name)
{
  marcel_t tid;
  marcel_attr_t attr;
  marcel_attr_init(&attr);
  marcel_attr_setdetachstate(&attr, MARCEL_CREATE_DETACHED);
  marcel_create(&tid, &attr, f, key);
  padico_tm_thread_givename_tid(name, tid);
}
