/** @file test Puk alone without PukABI and PadicoTM
 */

#include <Padico/Puk.h>

#include <Padico/Module.h>

PADICO_MODULE_BUILTIN(test_puk, NULL, NULL, NULL);

/* ********************************************************* */
/* *** Interface 'Dummy' */

struct padico_dummy_driver_s
{
  int (*dummy)(int);
};

PUK_IFACE_TYPE(Dummy, struct padico_dummy_driver_s);

/* ********************************************************* */
/* *** drivers */

static void*test_comp_instantiate(puk_instance_t ai, puk_context_t context);
static void test_comp_destroy(void*_instance);

const static struct puk_component_driver_s test_comp_component_driver =
  {
    .instantiate = &test_comp_instantiate,
    .destroy     = &test_comp_destroy
  };

static int test_comp_dummy(int a);

const static struct padico_dummy_driver_s test_comp_dummy_driver =
  {
    .dummy = &test_comp_dummy
  };


/* ********************************************************* */
/* *** implementation */

static int test_comp_dummy(int a) __attribute__((noinline));
static int test_comp_dummy(int a)
{
  return a;
}

static void*test_comp_instantiate(puk_instance_t instance, puk_context_t context)
{
  return padico_malloc(sizeof(int));
}

static void test_comp_destroy(void*_instance)
{
  free(_instance);
}

/* ********************************************************* */

int main(int argc, char**argv)
{
  puk_component_t component;
  puk_instance_t instance; 
  struct puk_receptacle_Dummy_s comp_receptacle;

  padico_puk_init(argc, argv);
  fprintf(stderr, "Hello world!\n");

  component = 
    puk_component_declare("TestComponents",
			puk_component_provides("PadicoComponent", "instance", &test_comp_component_driver),
			puk_component_provides("Dummy",         "dummy", &test_comp_dummy_driver),
			puk_component_attr("toto", "default_value"));
  instance = puk_component_instantiate(component);

  puk_component_resolve("TestComponents");
  puk_component_get_driver_Dummy(component, NULL);
  puk_instance_indirect_Dummy(instance, NULL, &comp_receptacle);

  puk_instance_destroy(instance);

  padico_puk_shutdown();
  return 0;
}

