/** @file bench-minidriver.c
 */

#include "bench-common.h"
#include <Padico/Module.h>
#include <Padico/Puk.h>
#include <Padico/AddrDB.h>
#include <Padico/NetSelector.h>

#include <nm_minidriver.h>

PADICO_MODULE_DECLARE(bench_minidriver);
PADICO_MODULE_INIT(bench_minidriver_init);
PADICO_MODULE_RUN(bench_minidriver_run);


static struct puk_receptacle_NewMad_minidriver_s r;

int bench_minidriver_init(void)
{
  /* find component */
  padico_na_node_t peer = bench_common_param.is_server ?
    bench_common_param.client_node :
    bench_common_param.server_node;
  const char*s_component = padico_getenv("BENCH_MINIDRIVER_ADAPTER");
  puk_component_t component = NULL;
  if(s_component != NULL)
    {
      if(puk_component_resolve(s_component) == NULL)
	padico_fatal("bench-minidriver: component %s not found.\n", s_component);
      char s_composite[512];
      snprintf(s_composite, 512,
	       "<puk:composite id=\"bench_minidriver\">"
	       "  <puk:component id=\"0\" name=\"%s\"/>"
	       "  <puk:entry-point iface=\"NewMad_minidriver\" port=\"minidriver\" provider-id=\"0\" />"
	       "</puk:composite>",
	       s_component);
      component = puk_component_parse(s_composite);
      padico_print("bench-minidriver: using component %s set manually.\n", s_component);
    }
  else
    {
      component = padico_ns_serial_selector(peer, "default", puk_iface_NewMad_minidriver());
      padico_print("bench-minidriver: using component %s from NetSelector.\n", component->name);
    }
  puk_context_t context = puk_component_get_context(component, puk_iface_NewMad_minidriver(), NULL);
  if(context == NULL)
    {
      padico_fatal("bench-minidriver: cannot find context in minidriver component. Cannot instantiate.\n");
    }
  /* init */
  const struct nm_minidriver_iface_s*driver = puk_component_get_driver_NewMad_minidriver(component, NULL);
  struct nm_minidriver_properties_s props; 
  props.profile.cpuset = hwloc_bitmap_alloc();
  (*driver->getprops)(-1, &props);
  const void*drv_url = NULL;
  size_t url_size = 0;
  (*driver->init)(context, &drv_url, &url_size);
  /* instantiate */
  puk_instance_t instance = puk_component_instantiate(component);
  puk_instance_indirect_NewMad_minidriver(instance, "minidriver", &r);
  /* connect */
  padico_addrdb_publish(peer, "bench-minidriver", NULL, 0, drv_url, url_size);
  char peer_url[url_size];
  padico_req_t req = padico_tm_req_new(NULL, NULL);
  padico_addrdb_get(peer, "bench-minidriver", NULL, 0, &peer_url[0], url_size, req);
  padico_tm_req_wait(req);
  (*r.driver->connect)(r._status, peer_url, url_size);
  padico_print("bench-minidriver: connected using %s\n", component->name);

  return 0;
}

void minidriver_server(char*buf, size_t len)
{
  struct iovec v = { .iov_base = buf, .iov_len = len };
  int rc = -1;
  (*r.driver->send_post)(r._status, &v, 1);
  do
    {
      rc = (*r.driver->send_poll)(r._status);
    }
  while(rc != 0);
  (*r.driver->recv_init)(r._status, &v, 1);
  do
    {
      rc = (*r.driver->poll_one)(r._status);
    }
  while(rc != 0);
}

void minidriver_client(char*buf, size_t len)
{
  struct iovec v = { .iov_base = buf, .iov_len = len };
  int rc = -1;
  (*r.driver->recv_init)(r._status, &v, 1);
  do
    {
      rc = (*r.driver->poll_one)(r._status);
    }
  while(rc != 0);
  (*r.driver->send_post)(r._status, &v, 1);
  do
    {
      rc = (*r.driver->send_poll)(r._status);
    }
  while(rc != 0);
}

static const struct bench_desc_s minidriver_bench =
{
  "minidriver",
  &minidriver_server,
  &minidriver_client
};

int bench_minidriver_run(int argc, char**argv)
{
  the_bench(&minidriver_bench);
  return 0;
}
