/** @file Launcher stub for 'LegacyBinary' service.
 */

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>

#include <Padico/Puk.h>
#include <Padico/PadicoTM.h>


extern void padico_legacy_start(void);

static void padico_legacy_init(void) __attribute__((constructor));
static void padico_legacy_finalize(void) __attribute__((destructor));

static struct
{
  marcel_mutex_t lock;
  marcel_cond_t event;
  int ready;
} legacy = { .ready = 0 };

/* ********************************************************* */

static void padico_legacy_init(void)
{
  if(getenv("PADICO_VERBOSE"))
    {
      fprintf(stderr, "LegacyBinary: starting...\n");
    }
  if(getenv("PADICO_VALGRIND"))
    {
      fprintf(stderr, "# PadicoTM: valgrind detected. No preinit done.\n");
      unsetenv("PADICO_VALGRIND");
#ifdef PUK_VG_NOOP
      fprintf(stderr, "# PadicoTM: WARNING- running on valgrind *without valgrind integration*.\n");
      sleep(3);
#endif
      return;
    }
  puk_abi_disable_preload();
  legacy.ready = 0;
  marcel_mutex_init(&legacy.lock, NULL);
  marcel_cond_init(&legacy.event, NULL);

  padico_puk_init(0, NULL);
  padico_tm_init();
  
  marcel_mutex_lock(&legacy.lock);
  while(!legacy.ready)
    {
      marcel_cond_wait(&legacy.event, &legacy.lock);
    }
  marcel_mutex_unlock(&legacy.lock);
}

static void padico_legacy_finalize(void)
{
  /*  padico_puk_shutdown(); */
  if(getenv("PADICO_VERBOSE"))
    {
      fprintf(stderr, "LegacyBinary: exit.\n");
    }
}


void padico_legacy_start(void)
{
  marcel_mutex_lock(&legacy.lock);
  legacy.ready = 1;
  marcel_cond_signal(&legacy.event);
  marcel_mutex_unlock(&legacy.lock);
}

