/** @file
 * @brief API for protocol selection in PadicoTM
 * @ingroup NetSelector
 */

#ifndef PADICO_NETSELECTOR_H
#define PADICO_NETSELECTOR_H

#include <Padico/Puk.h>
#include <Padico/Puk-hashtable.h>
#include <Padico/Module.h>
#include <Padico/PadicoTM.h>
#include <Padico/Topology.h>

/** @defgroup NetSelector Core module: NetSelector -- The PadicoTM network selector
 * @author Francois Lesueur
 * @author Alexandre Denis
 * @see @ref NetAccessAdapter
 */

#ifdef __cplusplus
extern "C" {
#endif

  /** @defgroup NetSelectorQuery API: NetSelector query and configure interface
   * @see @ref NetAccessAdapter
   * @ingroup NetSelector
   */

 
  /** @addtogroup NetSelectorQuery
   * @{
   */

  
  /** NetSelector driver interface
   */
  struct padico_netselector_driver_s
  {
    /** name of this NetSelector implementation */
    const char*name;

    /** name of the targeted profile */
    const char*profile;

    /** select a serial assembly towards a PadicoTM-enabled node */
    puk_component_t (*serial)(padico_na_node_t node1, padico_na_node_t node2,
			    const char*profile, 
			    puk_iface_t iface);

    /** select a serial assembly towards an abitrary host (PadicoTM-enabled or not) */
    puk_component_t (*host)(padico_na_host_t host, const char*profile, 
			  puk_iface_t iface, uint16_t port, int protocol);
  };

  PUK_IFACE_TYPE(NetSelector, struct padico_netselector_driver_s);

  void padico_ns_selector_register(const struct padico_netselector_driver_s*selector);


  /* *** Query to the selector ***************************** */
  
  puk_component_t padico_ns_serial_selector(padico_na_node_t node, 
					  const char*profile,
					  puk_iface_t iface);

  puk_component_t padico_ns_host_selector(padico_na_host_t host, 
					const char*profile,
					puk_iface_t iface, uint16_t port, int protocol);
  
  /** @} */

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* PADICO_NETSELECTOR_H */

