/** @file
 * @brief API for group management in PadicoTM NetAccess
 * @note should be used through PadicoControl.h
 * @ingroup NetAccess
 */

#ifndef PADICO_NA_GROUP_H
#define PADICO_NA_GROUP_H

#include <Padico/Puk.h>

/* *** Group ****************************************************** */

PUK_LIST_DECLARE_TYPE(padico_group);

padico_group_t   padico_group_init    (const char*id);
void             padico_group_publish (padico_group_t g);
int              padico_group_store   (padico_group_t g);
padico_group_t   padico_group_lookup  (const char*id);
void             padico_group_node_add(padico_group_t, padico_na_node_t);
void             padico_group_destroy (padico_group_t);
int              padico_group_size    (padico_group_t);
padico_na_node_t padico_group_node    (padico_group_t g, int rank);
const char*      padico_group_name    (padico_group_t g);
/** Rank of the given node in the group */
int              padico_group_rank    (padico_group_t, padico_na_node_t);
/** kill all nodes in group */
void             padico_group_kill    (padico_group_t g, int rc);
/** create a new group with nodes sorted according to given topology */
padico_group_t   padico_group_topology(padico_group_t g, const char*distrib);

/* ***** Group requests ************************************ */

typedef struct padico_group_req_s*padico_group_req_t;

padico_group_req_t padico_group_send_to_all(padico_group_t group, const char*cmd);
padico_rc_t        padico_group_wait_all   (padico_group_t group, padico_group_req_t greq);

#endif /* PADICO_NA_GROUP_H */
