/** @file
 * @brief CORBA header- public API for application using PadicoTM-enabled CORBA
 * @ingroup CORBA
 */

#ifndef PADICO_CORBA_H
#define PADICO_CORBA_H

#include <myCORBA.h>

#if defined(__cplusplus)

class Padico
{
public:
  static CORBA::ORB_ptr the_orb;
  class ORB 
  {
  public:
    typedef char*ObjectId;
#if 0
    typedef string ObjectId;
    typedef sequence <ObjectId> ObjectIdList;
    exception InvalidName {};
    // Dynamic Invocation related operations 
    void create_list ( in long count, out NVList new_list ); 
    void create_operation_list ( in OperationDef oper, out NVList new_list ); 
    void get_default_context ( out Context ctx ); 
    void send_multiple_requests_oneway( in RequestSeq req ); 
    void send_multiple_requests_deferred( in RequestSeq req ); 
    boolean poll_next_response(); 
    void get_next_response(out Request req ); 
    // Service information operations 
    boolean get_service_information ( in ServiceType service_type, out ServiceInformation service_information );
    ObjectIdList list_initial_services (); 
    // Type code creation operations 
    TypeCode create_struct_tc ( in RepositoryId id, in Identifier name, in StructMemberSeq members ); 
    TypeCode create_union_tc ( in RepositoryId id, in Identifier name, in TypeCode discriminator_type, in UnionMemberSeq members ); 
    TypeCode create_enum_tc ( in RepositoryId id, in Identifier name, in EnumMemberSeq members ); 
    TypeCode create_alias_tc ( in RepositoryId id, in Identifier name, in TypeCode original_type ); 
    TypeCode create_exception_tc ( in RepositoryId id, in Identifier name, in StructMemberSeq members );
    TypeCode create_interface_tc ( in RepositoryId id, in Identifier name ); 
    TypeCode create_string_tc ( in unsigned long bound );
    TypeCode create_wstring_tc ( in unsigned long bound ); 
    TypeCode create_fixed_tc ( in unsigned short digits, in short scale ); 
    TypeCode create_sequence_tc ( in unsigned long bound, in TypeCode element type );
    TypeCode create_recursive_sequence_tc (// deprecated 
					   in unsigned long bound, in unsigned long offset );
    TypeCode create_array_tc ( in unsigned long length, in TypeCode element_type );
    TypeCode create_value_tc ( in RepositoryId id, in Identifier name, in ValueModifier type_modifier, in TypeCode concrete_base, in ValueMembersSeq members ); 
    TypeCode create_value_box_tc ( in RepositoryId id, in Identifier name, in TypeCode boxed_type ); 
    TypeCode create_native_tc ( in RepositoryId id, in Identifier name );
    TypeCode create_recursive_tc( in RepositoryId id ); 
    TypeCode create_abstract_interface_tc( in RepositoryId id, in Identifier name ); 
    TypeCode create_local_interface_tc( in RepositoryId id, in Identifier name ); 
    // Policy related operations 
    Policy create_policy( in PolicyType type, in any val ) raises (PolicyError); 
    // Dynamic Any related operations deprecated and removed 
    // from primary list of ORB operations 
    // Value factory operations 
    ValueFactory register_value_factory( in RepositoryId id, in ValueFactory factory ); 
    void unregister_value_factory(in RepositoryId id); 
    ValueFactory lookup_value_factory(in RepositoryId id);
#endif
    // Thread related operations 
    CORBA::Boolean work_pending( )
    {
      return 1;
    }
    void perform_work()
    {
    }
    void run()
    {
    }
    void shutdown(CORBA::Boolean wait_for_completion )
    {
    }
    void destroy()
    {
    }
    char* object_to_string(CORBA::Object_ptr obj)
    {
      return Padico::the_orb->object_to_string(obj);
    }
    CORBA::Object_ptr string_to_object(const char* str )
    {
      return Padico::the_orb->string_to_object(str);
    }
    // Initial reference operation 
    CORBA::Object_ptr resolve_initial_references(const Padico::ORB::ObjectId identifier)
    {
      return Padico::the_orb->resolve_initial_references(identifier);
    }
 };
  typedef Padico::ORB* ORB_ptr;
  static  Padico::ORB_ptr ORB_init()
  {
    return new Padico::ORB();
  }
  static  Padico::ORB_ptr ORB_init(int& argc, char** argv,
				   const char* orb_identifier="")
  {
    return new Padico::ORB();
  }
};


#else
#error "Padico::ORB is C++ only!"
#endif

#endif /* PADICO_CORBA_H */


