/** @file lock-free stacks as chained list
 */

#ifndef PUK_LFSTACK_H
#define PUK_LFSTACK_H

static inline void puk_lfstack_backoff(int p)
{
  if(p > 20) p = 20;
  if(p > 2)
    {
      struct timespec t;
      t.tv_sec = 0;
      t.tv_nsec = 100 * p;
      nanosleep(&t, NULL);
    }
}

#define PUK_LFSTACK_TYPE(ENAME, DECL)					\
  struct ENAME ## _lfstack_cell_s					\
  {									\
    struct ENAME ## _lfstack_cell_s*_next;				\
    DECL								\
  };								        \
  typedef struct ENAME ## _lfstack_cell_s* ENAME ## _lfstack_t;		\
  static inline void ENAME ## _lfstack_push(ENAME ## _lfstack_t*_stack, struct ENAME ## _lfstack_cell_s*_cell) \
  {									\
    int r = 0;								\
    do									\
      {									\
	void*const _next = *_stack;					\
	_cell->_next = _next;						\
	if(!__sync_bool_compare_and_swap(_stack, _next, _cell))		\
	  { puk_lfstack_backoff(r++); }					\
	else								\
	  { r = 0; }							\
      }									\
    while(r != 0);							\
  }									\
  static inline struct ENAME ## _lfstack_cell_s* ENAME ## _lfstack_pop(ENAME ## _lfstack_t*_stack) \
  {									\
    int r = 0;								\
    struct ENAME ## _lfstack_cell_s*_cell = NULL;			\
    do									\
      {									\
	_cell = *_stack;						\
	if(_cell == NULL) return NULL;					\
	ENAME ## _lfstack_t _new_head = _cell->_next;			\
	if(!__sync_bool_compare_and_swap(_stack, _cell, _new_head))	\
	  { puk_lfstack_backoff(r++); }					\
	else								\
	  { r = 0; }							\
      }									\
    while(r != 0);							\
    _cell->_next = NULL;						\
    return _cell;							\
  }

#define puk_container_of(ptr, type, member)				\
        ((type *)((char *)(__typeof__ (&((type *)0)->member))(ptr)- ((size_t) &((type *)0)->member)))

#define PUK_ALLOCATOR_ALIGN 16

static inline void puk_allocator_cell_clear(void*ptr, size_t size)
{
#ifdef DEBUG
  memset(ptr, 0, size);
#endif
}
#ifdef DEBUG
#define PUK_ALLOCATOR_REDZONE_SIZE   8
#define PUK_ALLOCATOR_REDZONE(LABEL) char LABEL[PUK_ALLOCATOR_REDZONE_SIZE];
#else
#define PUK_ALLOCATOR_REDZONE(LABEL)
#define PUK_ALLOCATOR_REDZONE_SIZE   0
#endif

#define PUK_ALLOCATOR_TYPE(ENAME, TYPE)					\
  PUK_LFSTACK_TYPE(ENAME##_block, PUK_ALLOCATOR_REDZONE(_z1); TYPE block __attribute__((aligned(PUK_ALLOCATOR_ALIGN))); PUK_ALLOCATOR_REDZONE(_z2); ); \
  PUK_LFSTACK_TYPE(ENAME##_slice, struct ENAME##_block_lfstack_cell_s block; ); \
  struct ENAME##_allocator_s						\
  {									\
    ENAME##_block_lfstack_t blocks;					\
    ENAME##_slice_lfstack_t slices;					\
    int slab_size;							\
  };									\
  typedef struct ENAME##_allocator_s*ENAME##_allocator_t;  	   	\
  static inline ENAME##_allocator_t ENAME##_allocator_new(int slab_size) \
  {									\
    struct ENAME##_allocator_s*a = malloc(sizeof(struct ENAME##_allocator_s)); \
    a->blocks = NULL;							\
    a->slices = NULL;							\
    a->slab_size = slab_size;						\
    PUK_VG_CREATE_MEMPOOL(a, PUK_ALLOCATOR_REDZONE_SIZE, 0);		\
    return a;								\
  }									\
  static inline void ENAME##_allocator_delete(ENAME##_allocator_t a)	\
  {									\
    struct ENAME##_slice_lfstack_cell_s*slice = NULL;			\
    do {								\
      slice = ENAME##_slice_lfstack_pop(&a->slices);			\
      padico_free(slice);						\
    } while(slice != NULL);						\
    free(a);								\
    PUK_VG_DESTROY_MEMPOOL(a);						\
  }									\
  static inline TYPE*ENAME##_malloc(ENAME##_allocator_t a)		\
  {									\
    struct ENAME##_block_lfstack_cell_s*cell = ENAME##_block_lfstack_pop(&a->blocks); \
    if(cell == NULL)							\
      {									\
	int slab_size = a->slab_size;					\
	a->slab_size *= 2;						\
	struct ENAME##_slice_lfstack_cell_s*slice =			\
	  padico_malloc(sizeof(struct ENAME##_slice_lfstack_cell_s) + sizeof(struct ENAME##_block_lfstack_cell_s) * slab_size); \
	ENAME##_slice_lfstack_push(&a->slices, slice);			\
	struct ENAME##_block_lfstack_cell_s*block = &slice->block;	\
	cell = &block[0];						\
	int i;								\
	for(i = 1; i < slab_size; i++)					\
	  {								\
	    PUK_VG_MAKE_MEM_NOACCESS(&block[i].block, sizeof(TYPE));	\
	    ENAME##_block_lfstack_push(&a->blocks, &block[i]);		\
	  }								\
      }									\
    PUK_VG_MEMPOOL_ALLOC(a, &cell->block, sizeof(TYPE));		\
    puk_allocator_cell_clear(&cell->block, sizeof(TYPE));		\
    return &cell->block;						\
  }									\
  static inline void ENAME##_free(ENAME##_allocator_t a, TYPE*b)	\
  {									\
    struct ENAME##_block_lfstack_cell_s*cell = puk_container_of(b, struct ENAME##_block_lfstack_cell_s, block); \
    puk_allocator_cell_clear(&cell->block, sizeof(TYPE));		\
    PUK_VG_MEMPOOL_FREE(a, b);						\
    ENAME##_block_lfstack_push(&a->blocks, cell);			\
  }

#endif /* PUK_LFSTACK_H */
