dnl -*- mode: Autoconf;-*-

dnl -- Optionnal features
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_CHECK_ZLIB],
	 [
	   AC_CHECK_HEADER(zlib.h, 
 	        [ HAVE_ZLIB_H="yes"
	          HAVE_ZLIB="yes"
	          AC_DEFINE(HAVE_ZLIB_H, 1, [Define if you have the <zlib.h> header file.])],
	        [ HAVE_ZLIB_H="no"
	          HAVE_ZLIB="no" ])
	   AC_SUBST(HAVE_ZLIB)
	 ])

AC_DEFUN([AC_PADICO_CHECK_BZLIB],
	 [
	   AC_CHECK_HEADER(bzlib.h, 
	        [ HAVE_BZLIB_H="yes"
	          HAVE_BZLIB="yes"
	          AC_DEFINE(HAVE_BZLIB_H, 1, [Define if you have the <bzlib.h> header file.])],
	        [ HAVE_BZLIB_H="no"
	          HAVE_BZLIB="no" ])
	   AC_SUBST(HAVE_BZLIB)
 	 ])

AC_DEFUN([AC_PADICO_CHECK_LZ4],
	 [
	   AC_CHECK_HEADER(lz4.h, 
	        [ HAVE_LZ4_H="yes"
	          HAVE_LIBLZ4="yes"
	          AC_DEFINE(HAVE_LZ4_H, 1, [Define if you have the <lz4.h> header file.])],
	        [ HAVE_LZ4_H="no"
	          HAVE_LIBLZ4="no" ])
  	   AC_SUBST(HAVE_LIBLZ4)
	 ])

# ## checks for hwloc
# input: need_hwloc=yes|""
# output: with_hwloc, requires_hwloc, hwloc_CFLAGS|LIBS
AC_DEFUN([AC_PADICO_CHECK_HWLOC],
	 [
	   AC_REQUIRE([PKG_PROG_PKG_CONFIG])
	   AC_ARG_WITH([hwloc],
	               [AS_HELP_STRING([--with-hwloc], [use hwloc @<:@default=auto@:>@] )])
           if test "x${need_hwloc}" = "xyes"; then
  	     with_hwloc=yes
	   fi
	   HAVE_HWLOC=no
	   if test "x${with_hwloc}" != "xno"; then
 	     PKG_CHECK_MODULES([hwloc], [ hwloc ], [HAVE_HWLOC=yes], [HAVE_HWLOC=no])
	   fi
	   if test "x${with_hwloc}" = "xyes" -a "${HAVE_HWLOC}" = "no"; then
	     AC_MSG_ERROR([ required library hwloc not found.])
 	   fi
	   if test "${HAVE_HWLOC}" = "yes"; then
	     requires_hwloc=hwloc
	     AC_DEFINE([HAVE_HWLOC], [1], [whether hwloc is present])
	   fi
	   AC_SUBST([requires_hwloc])
	   AC_SUBST([with_hwloc])
	 ])

dnl --- Types
dnl --------------------------------------------------------

# ## socklen_t
AC_DEFUN(AC_TYPE_SOCKLEN_T,
[AC_CACHE_CHECK([for socklen_t], ac_cv_type_socklen_t,
[
  AC_TRY_COMPILE(
  [#include <sys/socket.h>],
  [socklen_t len = 42; return len;],
  ac_cv_type_socklen_t=yes,
  ac_cv_type_socklen_t=no)
])
  if test $ac_cv_type_socklen_t = yes; then
    AC_DEFINE(HAVE_SOCKLEN_T, [], [whether socklen_t is defined in sys/socket.h])
  fi
])

# ## off64_t
AC_DEFUN([AC_TYPE_OFF64_T],
	 [ AC_CACHE_CHECK([for off64_t], ac_cv_type_off64_t,
[
  AC_TRY_COMPILE(
  [#define _LARGEFILE64_SOURCE
   #include <sys/types.h>],
  [off64_t off = 42;],
  ac_cv_type_off64_t=yes,
  ac_cv_type_off64_t=no)
])
  if test $ac_cv_type_off64_t = yes; then
    AC_DEFINE(HAVE_OFF64_T, [], [whether off64_t is defined in sys/types.h using _LARGEFILE64_SOURCE])
    largefilecflags=-D_LARGEFILE64_SOURCE
  fi
  AC_SUBST(largefilecflags)
])

# ## check base required types
AC_DEFUN([AC_PADICO_TYPES],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])
	   AC_C_BIGENDIAN
	   AC_CHECK_SIZEOF(int)
	   AC_CHECK_SIZEOF(long)
	   AC_CHECK_SIZEOF(long long)
	   AC_CHECK_SIZEOF(ssize_t)
	   AC_TYPE_SOCKLEN_T
	   AC_TYPE_OFF_T
	   AC_TYPE_OFF64_T
	   AC_TYPE_SIZE_T
	   AC_TYPE_PID_T
	 ])

dnl --- libs
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_LIBS],
	 [ AC_REQUIRE([AC_PADICO_LINKER])
	   AC_SEARCH_LIBS(dlopen,  [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlopen() ]) ])
	   AC_SEARCH_LIBS(dlclose, [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlclose() ]) ])
	   AC_SEARCH_LIBS(dlsym,   [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlsym() ]) ])

	   AC_SEARCH_LIBS(sched_yield,   [c rt], [], [ AC_MSG_ERROR([Cannot find sched_yield() ]) ])

	   AC_SEARCH_LIBS(socket, nsl socket, [], [ AC_MSG_ERROR([Cannot find socket() ]) ])
	   AC_SEARCH_LIBS(listen, nsl socket, [], [ AC_MSG_ERROR([Cannot find listen() ]) ])
	   AC_SEARCH_LIBS(accept, nsl socket, [], [ AC_MSG_ERROR([Cannot find accept() ]) ])
	   AC_SEARCH_LIBS(gethostbyname, nsl socket, [], [ AC_MSG_ERROR([Cannot find gethostbyname() ]) ])
   
	 ])


dnl --- Arguments
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_OPTIMIZE],
	 [ ENABLE_OPTIMIZE_FLAG="yes";
	   AC_ARG_ENABLE(optimize,
	        	 [  --enable-optimize       Build with optimization compiler flags @<:@default: yes@:>@ ],
	                 [ if test $enableval = no; then
	                     ENABLE_OPTIMIZE_FLAG="no";
	                   fi
	                 ],
	                 [])
	   AC_SUBST(ENABLE_OPTIMIZE_FLAG)
	 ])

AC_DEFUN([AC_PADICO_DEBUG],
	 [ ENABLE_DEBUG_FLAG="no";
	   AC_ARG_ENABLE(debug,
	   	         [  --enable-debug          Build with debug flags @<:@default: no@:>@],
	                 [ if test $enableval = yes; then 
	                     ENABLE_DEBUG_FLAG="yes";
	                   fi
	                 ],
	                 [])
	   AC_SUBST(ENABLE_DEBUG_FLAG)
 	 ])

AC_DEFUN([AC_PADICO_TRACE],
	 [ AC_ARG_ENABLE(trace,
	                 [ --enable-trace ], 
	                 [ if test "x$enableval" = "xyes"; then
	                     AC_DEFINE(PADICO_TRACE_ON, 1, [enable Padico trace system])
	                   fi],
	                 [])
	 ])

